import { TranslateService } from '@ngx-translate/core';
import { AlertService } from './../services/alert.service';
import { AfterViewInit, Component, ElementRef, ViewChild } from '@angular/core';
import { AnimationController, GestureController, Platform, Animation } from '@ionic/angular';
import { Cell } from '../models/cell';
import { Share, ShareOptions } from '@capacitor/share';
import { AdMob, BannerAdPluginEvents, AdMobBannerSize, BannerAdOptions, BannerAdSize, BannerAdPosition } from '@capacitor-community/admob';

@Component({
  selector: 'app-game',
  templateUrl: 'game.page.html',
  styleUrls: ['game.page.scss'],
})
export class GamePage implements AfterViewInit {

  @ViewChild("boardGame", { read: ElementRef }) boardGame: ElementRef
  public board: Cell[][];

  private direction: number;
  private hasMovement: boolean;
  private isMoving: boolean;

  public points: number;
  private pointsRound: number;

  private DIRECTION_UP = 0;
  private DIRECTION_DOWN = 1;
  private DIRECTION_LEFT = 2;
  private DIRECTION_RIGHT = 3;

  public rows: number[];
  public cols: number[];

  public animations: Animation[];

  constructor(
    private gestureCtrl: GestureController,
    private alertService: AlertService,
    private translate: TranslateService,
    private animationCtrl: AnimationController,
    private platform: Platform
  ) {
    this.newGame();
    this.banner();
    this.isMoving = false;
    this.animations = [];
  }

  ngAfterViewInit(): void {

    // Movimiento horizontal
    const hSwipe = this.gestureCtrl.create(
      {
        el: this.boardGame.nativeElement,
        gestureName: 'hswipe',
        maxAngle: 30,
        direction: 'x',
        onEnd: (detail) => this.onHSwipe(detail)
      },
      true
    );


    // Movimiento vertical
    const vSwipe = this.gestureCtrl.create(
      {
        el: this.boardGame.nativeElement,
        gestureName: 'vswipe',
        maxAngle: 30,
        direction: 'y',
        onEnd: (detail) => this.onVSwipe(detail)
      },
      true
    );

    // Habilitamos los gestos
    vSwipe.enable();
    hSwipe.enable();

  }

  onHSwipe(detail) {

    // solo si no se esta moviendo
    if (!this.isMoving) {

      this.isMoving = true;

      console.log("Horizontal");
      console.log(detail);

      // si deltaX es menor que 0 significa que estas moviendote hacia la izquierda
      // sino significa que estas moviendote hacia la derecha
      if (detail.deltaX < 0) {
        console.log("Izquierda");
        this.direction = this.DIRECTION_LEFT;
        this.moveLeft();
      } else {
        console.log("Derecha");
        this.direction = this.DIRECTION_RIGHT;
        this.moveRight();
      }

      this.nextMove();

    }

  }


  onVSwipe(detail) {

    // solo si no se esta moviendo
    if (!this.isMoving) {

      this.isMoving = true;

      console.log("Vertical");
      console.log(detail);

      // si deltaY es menor que 0 significa que estas moviendote hacia arriba
      // sino significa que estas moviendote hacia abajo
      if (detail.deltaY < 0) {
        console.log("Arriba");
        this.direction = this.DIRECTION_UP;
        this.moveUp();
      } else {
        console.log("Abajo");
        this.direction = this.DIRECTION_DOWN;
        this.moveDown();
      }

      this.nextMove();

    }
  }

  nextMove() {

    // Si ganamos, nos muestra un mensaje
    if (this.winGame()) {
      this.alertService.alertCustomButtons(
        this.translate.instant('label.win.game.title'),
        this.translate.instant('label.game.content', { "points": this.points }),
        [
          {
            text: this.translate.instant('label.new.game'),
            handler: () => {
              this.newGame();
            }
          },
          {
            text: this.translate.instant('label.share'),
            handler: () => {
              this.sharePuntuation();
              this.newGame();
            }
          }
        ],
        false
      )
    }

    // Si perdemos, nos muestra un mensaje
    if (this.loseGame()) {
      this.alertService.alertCustomButtons(
        this.translate.instant('label.lose.game.title'),
        this.translate.instant('label.game.content', { "points": this.points }),
        [
          {
            text: this.translate.instant('label.new.game'),
            handler: () => {
              this.newGame();
            }
          },
          {
            text: this.translate.instant('label.share'),
            handler: () => {
              this.sharePuntuation();
            }
          }
        ],
        false
      )
    }

    // Si hay movimiento
    if (this.hasMovement) {
      // Generamos un número aleatorio
      this.generateRandomNumber();
      // Reseteamos la variable
      this.hasMovement = false;

      // Si ha habido puntos en la ronda, mostramos la animacion de los puntos
      if (this.pointsRound > 0) {
        this.showAnimationPoints();
        this.pointsRound = 0;
      }

      // animacion agrupada
      const animationGrouped = this.animationCtrl.create()
        .duration(100)
        .addAnimation(this.animations);

      animationGrouped.play();

      setTimeout(() => {
        // reseteo de la animacion
        animationGrouped.stop();
        this.animations = [];
      }, 100);

      // Espero para el siguiente movimiento
      setTimeout(() => {
        this.isMoving = false;
      }, 600);

    }else{
      this.isMoving = false;
    }

    // Limpiamos las celdas bloqueadas
    this.clearBlockedCells();
  }

  /**
   * Resetea el juego
   */
  newGame() {
    this.board = [
      [null, null, null, null],
      [null, null, null, null],
      [null, null, null, null],
      [null, null, null, null]
    ];
    this.rows = Array(4).fill(0);
    this.cols = Array(4).fill(0);
    this.hasMovement = false;
    this.points = 0;
    this.pointsRound = 0;
    this.generateRandomNumber();
    this.generateRandomNumber();
  }

  generateRandomNumber() {

    let row = 0;
    let col = 0;

    // Generamos una fila y columna aleatoria
    // Si existe algo en esa posicion, volvemos a pedirla
    do {
      row = Math.floor(Math.random() * this.board.length);
      col = Math.floor(Math.random() * this.board[0].length);
    } while (this.board[row][col] != null);

    this.board[row][col] = new Cell();

    // Generamos un numero entre 1 y 100
    const probNumber4 = Math.floor(Math.random() * 100) + 1;

    // si es menor de 25, generamos un 4
    if (probNumber4 <= 25) {
      this.board[row][col].value = 4;

      // Animacion para cambiar el fondo de la celda
      const animation = this.animationCtrl.create()
        .addElement(document.getElementById(row + '' + col))
        .duration(500)
        .fromTo('background', 'rgba(238, 228, 218, 0.35)', '#eee1c9');

      animation.play();

      setTimeout(() => {
        animation.stop();
      }, 500);

    } else {
      this.board[row][col].value = 2;

      // Animacion para cambiar el fondo de la celda
      const animation = this.animationCtrl.create()
        .addElement(document.getElementById(row + '' + col))
        .duration(500)
        .fromTo('background', 'rgba(238, 228, 218, 0.35)', '#eee4da');

      animation.play();

      setTimeout(() => {
        animation.stop();
      }, 500);

    }

  }

  moveLeft() {
    for (let i = 0; i < this.board.length; i++) {
      for (let j = 1; j < this.board[i].length; j++) {
        this.processPosition(i, j);
      }
    }
  }

  moveRight() {
    for (let i = 0; i < this.board.length; i++) {
      for (let j = this.board[i].length - 2; j >= 0; j--) {
        this.processPosition(i, j);
      }
    }
  }

  moveUp() {
    for (let i = 1; i < this.board.length; i++) {
      for (let j = 0; j < this.board[i].length; j++) {
        this.processPosition(i, j);
      }
    }
  }

  moveDown() {
    for (let i = this.board.length - 2; i >= 0; i--) {
      for (let j = 0; j < this.board[i].length; j++) {
        this.processPosition(i, j);
      }
    }
  }

  processPosition(i: number, j: number) {
    const n = this.board[i][j];
    if (n != null) {
      // Encontramos la siguiente posicion libre
      const nextPosition = this.nextPositionFree(i, j, n.value);

      if (nextPosition) {

        const row = nextPosition[0];
        const col = nextPosition[1];

        // Sino existe esa posicion, le creamos una celda
        if (!this.board[row][col]) {
          this.board[row][col] = new Cell();
        }

        // Si el valor de la posicion original y la posicion libre coincide, se combina y sumamos puntos
        if (n.value == this.board[row][col].value) {
          const points = n.value * 2;
          this.board[row][col].value = points;
          this.board[row][col].blocked = true;
          this.points += points;
          this.pointsRound += points;
        } else {
          this.board[row][col] = n;
        }

        // Eliminamos la posicion original
        this.board[i][j] = null;
        // Indicamos que ha habido movimiento
        this.hasMovement = true;

        // Calculamos el numero de celdas que nos hemos movido
        let numberCells;
        switch (this.direction) {
          case this.DIRECTION_RIGHT:
          case this.DIRECTION_LEFT:
            numberCells = col - j;
            break;
          case this.DIRECTION_UP:
          case this.DIRECTION_DOWN:
            numberCells = row - i;
            break;
        }

        console.log(numberCells);
        this.showAnimationMove(i, j, numberCells);

      }
    }
  }

  winGame() {
    for (let i = 0; i < this.board.length; i++) {
      for (let j = 0; j < this.board[i].length; j++) {
        const n = this.board[i][j];
        if (n != null && n.value == 2048) {
          return true;
        }
      }
    }
    return false;
  }

  loseGame() {
    for (let i = 0; i < this.board.length; i++) {
      for (let j = 0; j < this.board[i].length; j++) {
        const n = this.board[i][j];
        if (n != null) {

          if (
            (this.board[i - 1] && this.board[i - 1][j] && this.board[i - 1][j].value == n.value) ||
            (this.board[i][j + 1] && this.board[i][j + 1].value == n.value) ||
            (this.board[i + 1] && this.board[i + 1][j] && this.board[i + 1][j].value == n.value) ||
            (this.board[i][j - 1] && this.board[i][j - 1].value == n.value)
          ) {
            return false;
          }

        } else {
          return false;
        }
      }
    }
    return true;
  }

  clearBlockedCells() {
    for (let i = 0; i < this.board.length; i++) {
      for (let j = 0; j < this.board[i].length; j++) {
        const n = this.board[i][j];
        if (n != null) {
          n.blocked = false;
        }
      }
    }
  }

  nextPositionFree(rowOri: number, colOri: number, numberOriginal: number) {

    let rowNew: number;
    let colNew: number;
    let found: boolean = false;

    switch (this.direction) {
      case this.DIRECTION_LEFT:
        rowNew = rowOri;
        for (let j = colOri - 1; j >= 0 && !found; j--) {
          if (this.board[rowOri][j] != null) {
            found = true;

            if (this.board[rowOri][j].blocked) {
              colNew = j + 1;
            } else if (this.board[rowOri][j].value == numberOriginal) {
              colNew = j;
            } else if ((j + 1) != colOri) {
              colNew = j + 1;
            }
          }
        }

        if (!found) {
          colNew = 0;
        }

        break;
      case this.DIRECTION_RIGHT:
        rowNew = rowOri;
        for (let j = colOri + 1; j < this.board[rowOri].length && !found; j++) {
          if (this.board[rowOri][j] != null) {
            found = true;

            if (this.board[rowOri][j].blocked) {
              colNew = j - 1;
            } else if (this.board[rowOri][j].value == numberOriginal) {
              colNew = j;
            } else if ((j - 1) != colOri) {
              colNew = j - 1;
            }

          }
        }

        if (!found) {
          colNew = this.board[rowNew].length - 1;
        }

        break;
      case this.DIRECTION_UP:
        colNew = colOri;
        for (let i = rowOri - 1; i >= 0 && !found; i--) {
          if (this.board[i][colOri] != null) {
            found = true;

            if (this.board[i][colOri].blocked) {
              rowNew = i + 1;
            } else if (this.board[i][colOri].value == numberOriginal) {
              rowNew = i;
            } else if ((i + 1) != rowOri) {
              rowNew = i + 1;
            }
          }
        }

        if (!found) {
          rowNew = 0;
        }

        break;
      case this.DIRECTION_DOWN:
        colNew = colOri;
        for (let i = rowOri + 1; i < this.board.length && !found; i++) {
          if (this.board[i][colOri] != null) {
            found = true;

            if (this.board[i][colOri].blocked) {
              rowNew = i - 1;
            } else if (this.board[i][colOri].value == numberOriginal) {
              rowNew = i;
            } else if ((i - 1) != rowOri) {
              rowNew = i - 1;
            }
          }
        }

        if (!found) {
          rowNew = this.board.length - 1;
        }
        break;
    }

    if (
      (rowNew !== undefined && colNew !== undefined) &&
      (!this.board[rowNew][colNew] || this.board[rowNew][colNew].value === numberOriginal)
    ) {
      return [rowNew, colNew]
    }

    return null;
  }

  showAnimationPoints() {

    const pointsScored = document.getElementById('pointsScored');
    pointsScored.innerText = '+' + this.pointsRound;

    const animation = this.animationCtrl.create()
      .addElement(pointsScored)
      .duration(1000)
      .fromTo('transform', 'translateY(0px)', 'translateY(-60px)')
      .fromTo('opacity', 0, 1)

    animation.play();

    setTimeout(() => {
      animation.stop();
      pointsScored.innerText = '';
    }, 1000);

  }

  showAnimationMove(row: number, col: number, numberCells: number) {

    // animacion para el movimiento de la celda
    let animation = this.animationCtrl.create()
      .addElement(document.getElementById(row + '' + col));

    switch (this.direction) {
      case this.DIRECTION_RIGHT:
      case this.DIRECTION_LEFT:
        animation = animation.fromTo('transform', 'translateX(0px)', 'translateX(' + numberCells * 60 + 'px)');
        break;
      case this.DIRECTION_UP:
      case this.DIRECTION_DOWN:
        animation = animation.fromTo('transform', 'translateY(0px)', 'translateY(' + numberCells * 60 + 'px)');
        break;
    }

    // Agregamos la animacion al grupo de animaciones
    this.animations.push(animation);

  }

  async sharePuntuation() {

    const shareOptions: ShareOptions = {
      title: '2048',
      text: this.translate.instant('label.share.dialog.title', { points: this.points }),
      dialogTitle: this.translate.instant('label.share.dialog.title', { points: this.points })
    };

    // Depende de la plataforma usada, cambiamos la url
    if (this.platform.is('android')) {
      shareOptions.url = 'https://play.google.com/store';
    } else if (this.platform.is('ios')) {
      shareOptions.url = 'https://apple.com/es/app-store';
    }

    // Aparece el mensaje para compartir
    await Share.share(shareOptions);
  }

  banner() {

    // Listener con eventos de nuestros anuncios
    AdMob.addListener(BannerAdPluginEvents.Loaded, () => {
      // Subscribe Banner Event Listener
      console.log("Cargado los anuncios");
    });

    AdMob.addListener(BannerAdPluginEvents.SizeChanged, (size: AdMobBannerSize) => {
      // Subscribe Change Banner Size
      console.log("Tamaño cambiado: ", size);
    });

    // Segun la plataforma, mostramos un banner u otro
    let options: BannerAdOptions;
    if (this.platform.is('android')) {
      options = {
        adId: 'ca-app-pub-4695431720501222/2084606895',
        adSize: BannerAdSize.BANNER,
        position: BannerAdPosition.TOP_CENTER,
        margin: 0,
        isTesting: true
      };
    } else if (this.platform.is('ios')) {
      options = {
        adId: 'ca-app-pub-4695431720501222/6182390737',
        adSize: BannerAdSize.BANNER,
        position: BannerAdPosition.TOP_CENTER,
        margin: 0,
        isTesting: true
      };
    }

    // Mostramos el banner
    AdMob.showBanner(options);
  }

}
